@@ -114,6 +114,8 @@ GST_DEBUG_CATEGORY_STATIC (clutter_gst_video_sink_debug);
 static const char clutter_gst_video_sink_caps_str[] =
   MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY, BASE_SINK_CAPS)
 #ifdef HAVE_GL_TEXTURE_UPLOAD
+  ";"
+  MAKE_CAPS_COMPOSITON (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA }")
   ";"
   MAKE_CAPS (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA }")
 #endif
@@ -1764,9 +1766,23 @@ append_cap (gpointer data,
 {
   ClutterGstRenderer *renderer = (ClutterGstRenderer *) data;
   GstCaps *caps = (GstCaps *) user_data;
+  guint i, count;
+  GstCapsFeatures *features;
   GstCaps *writable_caps;
+  GstCaps *writable_caps_with_composition;
+
   writable_caps =
     gst_caps_make_writable (gst_static_caps_get (&renderer->caps));
+
+  writable_caps_with_composition = gst_caps_copy (writable_caps);
+  count = gst_caps_get_size (writable_caps_with_composition);
+
+  for (i = 0; i < count; i++) {
+    features = gst_caps_get_features (writable_caps_with_composition, i);
+    gst_caps_features_add (features, GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION);
+  }
+
+  gst_caps_append (caps, writable_caps_with_composition);
   gst_caps_append (caps, writable_caps);
 }
 
@@ -1812,7 +1828,7 @@ clutter_gst_video_sink_get_caps (GstBaseSink *bsink,
   ClutterGstVideoSink *sink;
   sink = CLUTTER_GST_VIDEO_SINK (bsink);
 
-  GST_DEBUG_OBJECT (bsink, "Getting caps for %s", gst_caps_to_string (filter));
+  GST_DEBUG_OBJECT (bsink, "Getting caps for %" GST_PTR_FORMAT, filter);
 
   if (sink->priv->caps == NULL)
     return NULL;
@@ -1940,7 +1956,7 @@ clutter_gst_video_sink_set_caps (GstBaseSink *bsink,
   sink = CLUTTER_GST_VIDEO_SINK (bsink);
   priv = sink->priv;
 
-  GST_INFO_OBJECT (bsink, "Setting caps to %s", gst_caps_to_string (caps));
+  GST_INFO_OBJECT (bsink, "Setting caps to %" GST_PTR_FORMAT, caps);
 
   if (!clutter_gst_video_sink_parse_caps (caps, sink, FALSE))
     return FALSE;
@@ -2289,6 +2305,31 @@ clutter_gst_video_sink_propose_allocation (GstBaseSink *base_sink, GstQuery *que
   return TRUE;
 }
 
+static gboolean
+clutter_gst_video_sink_event (GstBaseSink * basesink, GstEvent * event)
+{
+  ClutterGstVideoSink *sink = CLUTTER_GST_VIDEO_SINK (basesink);
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  ClutterGstSource *gst_source = priv->source;
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
+      g_mutex_lock (&gst_source->buffer_lock);
+      if (gst_source->buffer) {
+        GST_DEBUG ("Freeing existing buffer %p", gst_source->buffer);
+        gst_buffer_unref (gst_source->buffer);
+        gst_source->buffer = NULL;
+      }
+      g_mutex_unlock (&gst_source->buffer_lock);
+      break;
+
+    default:
+      break;
+  }
+
+  return GST_BASE_SINK_CLASS (clutter_gst_video_sink_parent_class)->event (basesink, event);
+}
+
 static void
 clutter_gst_video_sink_class_init (ClutterGstVideoSinkClass *klass)
 {
@@ -2330,6 +2371,7 @@ clutter_gst_video_sink_class_init (ClutterGstVideoSinkClass *klass)
   gb_class->set_caps = clutter_gst_video_sink_set_caps;
   gb_class->get_caps = clutter_gst_video_sink_get_caps;
   gb_class->propose_allocation = clutter_gst_video_sink_propose_allocation;
+  gb_class->event = clutter_gst_video_sink_event;
 
   gv_class->show_frame = _clutter_gst_video_sink_show_frame;
 
